use futures::stream::{BoxStream, StreamExt as _};
use juniper::{graphql_subscription, FieldError};
use std::time::Duration;
use tokio::time::interval;
use tokio_stream::wrappers::IntervalStream;

use crate::database::Database;

#[derive(Clone, Copy, Debug)]
pub struct Subscription;

type NumberStream = BoxStream<'static, Result<i32, FieldError>>;

#[graphql_subscription(context = Database)]
impl Subscription {
    /// Counts seconds.
    async fn count() -> NumberStream {
        let mut value = 0;
        let stream = IntervalStream::new(interval(Duration::from_secs(1))).map(move |_| {
            value += 1;
            Ok(value)
        });
        Box::pin(stream)
    }
}
